<?php
/* 
	simple_api.php 
	Created: 05.24.2010 by Jason "pixelat3d" snively pixelat3d@siliconfreaks.com
	Last Updated: 
*/

class data_slurp {
	protected $username;
	protected $password;
	protected $format;
	private	$version = 0.1;

	public function __construct( $username, $password = '', $format = 'json' ) {
		$this->username = $username;
		$this->password = $password;
		$this->format = $format;
	}

	final protected function request_data( $url, $format = 'json' ) {
		$ch	= curl_init( $url );
	
		// If requred auth, then auth. 	
		if ( !empty( $this->username ) && !empty( $this->password ) ) {
			curl_setopt( $ch, CURLOPT_USERPWD, $this->username.':'.$this->password);
		}

		curl_setopt( $ch, CURLOPT_VERBOSE, 1 );
		curl_setopt( $ch, CURLOPT_NOBODY, 0 );
		curl_setopt( $ch, CURLOPT_HEADER, 0 );
		curl_setopt( $ch, CURLOPT_TIMEOUT, 3 );
		curl_setopt( $ch, CURLOPT_USERAGENT, 'siliconfreaks-slurp/'.$this->version );
		curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, 1 );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );

		$response 	= curl_exec( $ch );
		$r_info		= curl_getinfo( $ch );
		curl_close( $ch );

		if( $r_info['http_code'] == 200 ) {

			switch( strtolower( $this->format ) ) {
				case 'json': return ( json_decode( $response ) ); break;
				case 'xml': return( $response ); break;
				default: return( $response ); break;
			}

			return( $r_info['http_code'] );
		} else {
			return ( "Error # $r_info[http_code]" );
		}
	}

	public function get_user( ) { return( $this->username ); }
	public function set_user( $username ) { $this->username = $username; }

	public function get_format( ) { return( $this->format ); }
	public function set_format( $format ) { $this->format = $format; }

	public function set_password( $password ) { $this->password = $password; }
}

class steam_data extends data_slurp {
	private		$user_id = 0;
	private 	$friends 	= Array( Array( 'username' => '', 'id' => 0 ) );
	private		$games		= Array( Array( 'game' => '', 'id' => 0, 'hours' => 0, 'store' => '', 'stats' => '' ) );
	private		$private	= false;

	function __construct( $username ) { 

		$username = urldecode( $username );
		if( !is_numeric( $username ) ) {
			$this->username = $username;
			$this->user_id = $this->username_to_id( );
		} else {
			$this->user_id = $username;
			$this->username = $this->id_to_username( $this->user_id );
		}

		$this->get_friends( );
		$this->get_games( );
	}

	private function username_to_id( ) {
		// This function could be a lot better. 
		$string_s = 'steam://friends/add';
		$string_e = "\n";

		$blob = $this->request_data( 'http://steamcommunity.com/id/'.$this->username );
		if( stristr( $blob, '"steam://friends/add' ) ) {
			$start = strpos( $blob, 'steam://friends/add/' );
			$end = strpos( $blob, '"', $start );
			$link = substr( $blob, $start, ($end - $start)  );
			
			$link = explode( '/', $link );
	
			return( $link[4] );
		} else {
			//echo 'could not find user with custom URL, trying search...';
			$blob = $this->request_data( 'http://steamcommunity.com/actions/Search?T=Account&K='.$this->username, 'xml' );
			preg_match_all( '/\<a class=\"linkTitle\"+\shref=\"(.+)\"\>(.+)\<\/a\>/i', $blob, $search_results );

			if( count( $search_results[0] ) > 0 ) {
				//First result should be spot on if it matches account name.
				$uid = explode( '/', $search_results[1][0] );
				return( $uid[4] );

				/* Display search results and let user choose. 
				echo "	<br />Hrmm... we've got a guess. We're going to use the first one.<br />";
				for( $i = 0; $i < count( $search_results[0] ); $i++ ) {
					$uid = explode( '/', $search_results[1][$i] );
					//echo 'user: <a href="?user='.$uid[4].'">'.(string)$search_results[2][$i]."</a><br />";
					return( $uid[4] );
				} */
			} else {
				die( 'Invalid Account, User or whatever name.' );
			}
		}
	}

	private function id_to_username( $id ) {
		$xml = @simplexml_load_string( $this->request_data( 'http://steamcommunity.com/profiles/'.$id.'/?xml=1', 'xml' ) );
		
		if( $xml ) {
			return( (string)utf8_decode( $xml->children( )->steamID ) );
		}
	}

	private function get_friends( ) {
		$xml = @simplexml_load_string( $this->request_data( 'http://steamcommunity.com/profiles/'.$this->user_id.'/friends/?xml=1', 'xml' ) );
	
		if( !( $xml === false ) ) {	
			if( !$xml->error ) {
				$i = 0;
				foreach( @$xml->children( )->friends->friend as $friend ) {
					$this->friends[$i]['username'] = $this->id_to_username( $friend ); 
					$this->friends[$i]['id'] = (string)$friend;
					$i++;
				}
			}
		}
	}

	private function get_games( ) { 
		$xml = @simplexml_load_string( $this->request_data( 'http://steamcommunity.com/profiles/'.$this->user_id.'/games/?xml=1', 'xml' ) );

		if( !( $xml === false ) ) { 
			if( !$xml->error  ) {
				$i = 0;
				foreach( $xml->children( )->games->game as $game ) {
					$this->games[$i]['game'] 	= (string)utf8_decode( $game->name );
					$this->games[$i]['id']		= (string)$game->appID; 
					$this->games[$i]['hours']	= (string)$game->hoursOnRecord;
					$this->games[$i]['store']	= (string)$game->storeLink; 
					$this->games[$i]['stats']	= (string)$game->statsLink;
					$i++; 
				}
			}
		}
	}

	private function get_achievements( ) {
		//TODO: This.
	}

	public function games_list( ) { return( $this->games ); }
	public function friends_list( ) { return( $this->friends ); }
	public function whoami( ) { return( $this->username ); }
	public function friend_count( ) { return( count( $this->friends ) ); }
	public function game_count( ) {	return( count( $this->games ) ); }
}

	// Enhance!
	if( !empty( $_REQUEST['user'] ) ) {
		$steam_engine = new steam_data( $_REQUEST['user'] );
	} else {
		echo "Invalid Username, SteamId, or Search Term";
		die( );
	}
?>

<html>
	<style>
		body { color: #FFFFFF; background-color: #4C5844; font-family: Helvetica, Arial, sans-serif; }
		a{ color: #C4B550; border-bottom: 1px solid #FFF; text-decoration: none; }
		h1, h2 { background-color: #3e4637; padding: 10px; border-bottom: 1px solid #889180; border-right: 1px solid #889180; border-top: 1px solid #282e22; border-left: 1px solid #282e22; }
	</style>
	<title>Steam Friends and Games List</title>
	<body>
		<h1>Steam Info for <br /><a href="http://steamcommunity.com/id/<?=$steam_engine->whoami();?>/home"><?=$steam_engine->whoami( );?></a></h1>
		<h2>Friends List (<?=$steam_engine->friend_count( );?> total)</h2>
		<? include( 'friends.php' ); ?>

		<h2>Games List (<?=$steam_engine->game_count( );?> total)</h2>
		<? include( 'games.php' ); ?>
	</body>
</html>
